/* *******************************************************************
   PWM - Dimming with the ESP8266

   Hardware
   - H801 Dimmer
   - NodeMCU / WEMOS D1 or any other generic ESP8266

   Short
   - simple webserver with some pages
   - Stylesheet (css) optimized for mobile devices
   - webseite update with fetch API
   - control pins via webpage
   - ArduinoOTA upload with Arduino IDE
   - see the full tutorial on: https://werner.rothschopf.net/

   Open Tasks
   - update the checkbox via JavaScript

   by noiasca:

   Version
   2021-06-21 (compiles with ESP8266 core 2.7.4 without warnings)
***************************************************************** */

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>          // for the webserver
#include <ESP8266mDNS.h>               // Bonjour/multicast DNS, finds the device on network by name
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE

#define VERSION "1.0.0"                // the version of this sketch

//#include <credentials.h>               // my credentials - remove before upload
#include "general.h"                   // some general definitions (nothing to change)

/* *******************************************************************
         define which board variant to be compiled:
 ********************************************************************/

#define USE_BOARD 70                   // the actual board to compile

/* *******************************************************************
         the board settings / die Einstellungen der verschiedenen Boards
 ********************************************************************/

#if USE_BOARD == 70                    // example board: H801 Dimmer
#define TXT_BOARDNAME "H801 Dimmer"    // the name of the board
HardwareSerial &Terminal = Serial1;    // if you need debug messages, you have to send them to Serial1

//declare the pins                     // on the H801 dimmer:
const byte pwmRedPin = 15;             // GPIO15 red
const byte pwmGreenPin = 13;           // GPIO13 green
const byte pwmBluePin = 12;            // GPIO12 blue
const byte pwmW1Pin = 14;              // GPIO14 white warm
const byte pwmW2Pin = 4;               // GPIO04 white cold
const byte ledGreenPin = 1;            // GPIO01/D10 the green SMD LED on the PCB (left top corner)
const byte ledRedPin = 5;              // GPIO05/D1 the red SMD LED on the PCB (left bottom corner)
const byte flashPin = 0;               // GPIO00/D3 flash Jumper
// RX Serial = 03                      // as header available - can be used for sketch upload
// TX Serial1 = 02                     // as header available - but Serial1 (!)

// assign pins to the struct arrays:
// The H801 has only the flashPin exposed for the program jumper:
Pin inputPin[] {
  // the pin    a name for the website
  {flashPin,    "Jumper"},
};

// there are two LEDs on the PCB which can be used to indicate states of your program:
OutputPin outputPin[] {
  // the pin     a name          LOW active (or high active)
  {ledRedPin,   "onboard red",   LOW},
  {ledGreenPin, "onboard green", LOW}
};

// the H801 offers 5 PWM outputs connected to MOSFETS:
PWMPin pwmPin[] {
  // the pin     a name          a PWM value
  {pwmRedPin,   "red",           0},
  {pwmGreenPin, "green",         0},
  {pwmBluePin,  "blue",          0},
  {pwmW1Pin,    "warm white",    0},
  {pwmW2Pin,    "cold white",    0}
};
#endif

#if USE_BOARD == 99                    // example board: NodeMCU
#define TXT_BOARDNAME "NodeMCU"        // the name of the board
HardwareSerial &Terminal = Serial;

const uint8_t flashPin = 0;            // GPIO00/D3 on NodeMCU the flash Button - this is for testing an input
const uint8_t ouputAPin = 16;          // GPIO16/D0 on NodeMCU is a (somtimes red, sometimes blue) LED on the NodeMCU Board - use this for testing the html switch
const uint8_t ouputBPin = 2;           // GPIO02/D4 on NodeMCU is the (blue) LED on the ESP-12E

// assign pins to arrays:
Pin inputPin[] {
  // the pin    a name for the website
  {flashPin,    "Flash button"},
};

OutputPin outputPin[] {
  // the pin     a name          LOW active (or high active)
  {ouputAPin,   "outputPin 0",   LOW},
};

PWMPin pwmPin[] {
  // the pin     a name          a PWM value
  {ouputBPin,   "pwmPin 0",        0}
};

#endif

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/
#ifndef STASSID                        // either use an external .h file containing STASSID and STAPSK - or 
//                                     // add defines to your boards - or
#define STASSID "your-ssid"            // ... modify these line to your SSID
#define STAPSK  "your-password"        // ... and set your WIFI password
#endif

const char* ssid = STASSID;
const char* password = STAPSK;

/* *******************************************************************
         Globals - Variables and constants
 ********************************************************************/

unsigned long ss = 0;                  // current second since startup
const uint16_t ajaxIntervall = 5;      // intervall for AJAX or fetch API call of website in seconds

ADC_MODE(ADC_VCC);                     // to use getVcc
uint32_t internalVcc = ESP.getVcc();   // this will be used to measure the internal voltage

ESP8266WebServer server(80);           // an instance for the webserver

#ifndef CSS_MAINCOLOR
#define CSS_MAINCOLOR "#8A0829"        // fallback if no CSS_MAINCOLOR was declared for the board
#endif

/* *******************************************************************
         S E T U P
 ********************************************************************/

void setup(void) {
  // we start with the hardware pins
  for (auto &i : inputPin) pinMode(i.pin, INPUT_PULLUP);
  for (auto &i : pwmPin) pinMode(i.pin, OUTPUT);
  for (size_t i = 0; i < sizeof(outputPin) / sizeof(outputPin[0]); i++)
  {
    digitalWrite(outputPin[i].pin, !outputPin[i].ON); // switch off output
    pinMode(outputPin[i].pin, OUTPUT);
  }
  for (size_t i = 0; i < sizeof(pwmPin) / sizeof(pwmPin[0]); i++)
  {
    pwmPin[i].value = random(1023 + 1);
    analogWrite(pwmPin[i].pin, pwmPin[i].value);
  }
  Terminal.begin(115200);
  Terminal.print(F("\n" TXT_BOARDNAME "\nVersion: " VERSION " Board "));
  Terminal.println(USE_BOARD);
  Terminal.print  (__DATE__);
  Terminal.print  (F(" "));
  Terminal.println(__TIME__);

  char myhostname[16] {"esp"};         // you could use your TXT_BOARDNAME also, but I prefer to have short names
  char val[8];
  itoa(USE_BOARD, val, 10);            // before we can concatenate the precompiler integer, we have to convert it to char array
  strcat(myhostname, val);
  WiFi.hostname(myhostname);
  WiFi.begin(ssid, password);
  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Terminal.print(F("."));
    digitalWrite(outputPin[0].pin, !digitalRead(outputPin[0].pin)); // blink one pin as indicator
  }
  Terminal.println(F(""));
  Terminal.print(F("Connected to "));
  Terminal.println(ssid);
  Terminal.print(F("IP address: "));
  Terminal.println(WiFi.localIP());
  if (MDNS.begin("esp8266")) {
    Terminal.println(F("MDNS responder started"));
  }
  digitalWrite(outputPin[0].pin, outputPin[0].ON);
  //define the pages and other content for the webserver
  server.on("/",      handlePage);               // send root page
  server.on("/0.htm", handlePage);               // a request can reuse another handler
  server.on("/1.htm", handlePage1);              // a request can reuse another handler
  server.on("/f.css", handleCss);                // a stylesheet
  server.on("/j.js",  handleJs);                 // javscript based on fetch API to update the page
  server.on("/json",  handleJson);               // send data in JSON format
  server.on("/c.php", handleCommand);            // process commands
  server.on("/favicon.ico", handle204);          // not available, send 204
  server.onNotFound(handleNotFound);             // show a typical HTTP Error 404 page

  server.begin();                                // start the webserver
  Terminal.println(F("HTTP server started\n"));

  //IDE OTA
  ArduinoOTA.setHostname(myhostname);            // give a name to your ESP for the Arduino IDE
  ArduinoOTA.begin();                            // OTA Upload via ArduinoIDE https://arduino-esp8266.readthedocs.io/en/latest/ota_updates/readme.html
}

/* *******************************************************************
         M A I N L O O P
 ********************************************************************/

void loop(void) {
  ss = millis() / 1000;
  server.handleClient();               // call the webserver
  ArduinoOTA.handle();                 // OTA Upload via ArduinoIDE
  // do your calculation, sensor readings etc...

  // add displays, serial outputs here
}
